package pwc.taxtech.atms.controller;

import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import pwc.taxtech.atms.dto.navtree.DevTreeDto;
import pwc.taxtech.atms.dto.navtree.RolePermissionDisplayDto;
import pwc.taxtech.atms.service.impl.PermissionServiceImpl;

import java.util.List;

@RestController
@RequestMapping("/api/v1/permission/")
public class PermissionController {

    private static Logger logger = LoggerFactory.getLogger(PermissionController.class);

    @Autowired
    private PermissionServiceImpl permissionService;

    @ApiOperation(value = "Active or Deactive the specific area.")
    @RequestMapping(value = "getIvhTreePermissionsByRoleID", method = RequestMethod.GET)
    public @ResponseBody
    RolePermissionDisplayDto getIvhTreePermissionsByRoleId(@RequestParam String roleID,
                                                           @RequestParam String serviceType) {
        logger.info("getIvhTreePermissionsByRoleId: roleId={}, serviceType={}.", roleID, serviceType);
        return permissionService.getIvhTreePermissionsByRoleId(roleID, serviceType);
    }

    @ApiOperation(value = "通过传入的角色Id列表获取树形结构权限")
    @RequestMapping(value = "getIvhTreePermissionsByRoleIDList", method = RequestMethod.POST)
    public @ResponseBody
    List<DevTreeDto> getIvhTreePermissionsByRoleIdList(@RequestBody List<String> roleIdList,
                                                       @RequestParam String serviceType) {
        return permissionService.getIvhTreePermissionsByRoleIdList(roleIdList, serviceType);
    }

    @ApiOperation(value = "获取所有权限树形结构列表")
    @RequestMapping(value = "getAllPermissions", method = RequestMethod.GET)
    public @ResponseBody
    List<DevTreeDto> getAllPermissions(@RequestParam String serviceType) {
        return permissionService.getAllPermissions(serviceType);
    }
}
