package pwc.taxtech.atms.dto.vatdto;

import lombok.Getter;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.EvaluationException;
import org.apache.poi.ss.formula.eval.ValueEval;

import java.util.ArrayList;
import java.util.List;

import static pwc.taxtech.atms.common.util.FormulaUtil.resolverInteger;
import static pwc.taxtech.atms.common.util.FormulaUtil.resolverString;
import static pwc.taxtech.atms.exception.Exceptions.BAD_BBVO_PARAMS;

@Getter
public class BBParasBo {
    private String reportCode;
    private Integer columnIndex;
    private Integer rowIndex;
    private Integer period;
    private Integer year;
    private String formulaExpression;
    private List<PeriodCellDataTemplate> expressionData = new ArrayList<>();

    public static class PeriodCellDataTemplate {
        Integer period;
        Long cellTemplateId;

        @Override
        public String toString() {
            return period + ":" + cellTemplateId;
        }

        public PeriodCellDataTemplate(Integer period, Long cellTemplateId) {
            this.period = period;
            this.cellTemplateId = cellTemplateId;
        }

    }

    public void putPeriodCellTempate(Integer period, Long cellTemplateId) {
        if (period < 1 || period > 12 || cellTemplateId < 0) throw BAD_BBVO_PARAMS;
        expressionData.add(new PeriodCellDataTemplate(period, cellTemplateId));
    }

    public BBParasBo(ValueEval[] args, OperationEvaluationContext ec) throws EvaluationException {
        StringBuilder expression = new StringBuilder("");
        begin(expression);
        reportCode = resolverString(args, ec, 0);
        concatPara(expression, reportCode);
        try {
            columnIndex = resolverInteger(args, ec, 1);
            split(expression);
            concatPara(expression, columnIndex);
        } catch (Exception e) {
            String columnStr = resolverString(args, ec, 1);
            split(expression);
            concatPara(expression, columnStr);
            int rtn = 0;
            columnStr = columnStr.toUpperCase();
            char[] excelCol = columnStr.toCharArray();
            for (int i = columnStr.length() - 1; i >= 0; i--) {
                if (excelCol[i] >= 'A' && excelCol[i] <= 'Z') {
                    rtn += ((int) Math.pow(26, excelCol.length - 1 - i) * (excelCol[i] - 64));
                } else {
                    rtn = -1;
                }
            }
            columnIndex = rtn;
        }
        rowIndex = resolverInteger(args, ec, 2);
        split(expression);
        concatPara(expression, rowIndex);
        period = args.length >= 4 ? resolverInteger(args, ec, 3) : 0;
        year = args.length == 5 ? resolverInteger(args, ec, 4) : 0;
        if (args.length >= 4) {
            split(expression);
            concatPara(expression, period);
        }
        if (args.length == 5) {
            split(expression);
            concatPara(expression, year);
        }
        end(expression);
        formulaExpression = expression.toString();
    }

    private StringBuilder begin(StringBuilder expression) {
        return expression.append("BB(");
    }

    private StringBuilder end(StringBuilder expression) {
        return expression.append(")");
    }

    private StringBuilder split(StringBuilder expression) {
        return expression.append(",");
    }

    private StringBuilder concatPara(StringBuilder expression, String para) {
        return expression.append("\"").append(para).append("\"");
    }

    private StringBuilder concatPara(StringBuilder expression, Integer para) {
        return expression.append(para);
    }

    public BBParasBo(BBParasBo otherBo, int period, int curYear) {
        this.reportCode = otherBo.reportCode;
        this.columnIndex = otherBo.getColumnIndex();
        this.rowIndex = otherBo.rowIndex;
        this.period = period;
        this.year = curYear;
        this.formulaExpression = otherBo.formulaExpression;
    }

    public String expression() {
        return formulaExpression;
    }

    @Override
    public String toString() {
        return "BBParasBo{" +
                "reportCode='" + reportCode + '\'' +
                ", columnIndex=" + columnIndex +
                ", rowIndex=" + rowIndex +
                ", period=" + period +
                ", year=" + year +
                '}';
    }
}
