package pwc.taxtech.atms.dto.vatdto;

import io.jsonwebtoken.lang.Collections;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.StringUtils;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.vat.entity.PeriodJob;

import java.util.Date;
import java.util.List;

import static pwc.taxtech.atms.dto.vatdto.WrapPeriodJobDto.*;

@Getter
@Setter
public class PeriodJobDto {
    private String id;
    private String name;
    private String currentStep;
    private String projectId;
    private Integer period;
    private String stepsCode;
    private Date createTime;
    private String status;
    private String errorMsg;

    private String jobStatus;

    public PeriodJobDto copyFromPeriodJob(PeriodJob job) {
        this.id = job.getId();
        this.name = job.getName();
        this.currentStep = job.getCurrentStep();
        this.projectId = job.getProjectId();
        this.period = job.getPeriod();
        this.stepsCode = job.getStepsCode();
        this.createTime = job.getCreateTime();
        this.status = job.getStatus();
        this.errorMsg = job.getErrorMsg();
        MyAsserts.assertNotEmpty(stepsCode, Exceptions.SERVER_ERROR_EXCEPTION);

        if (StringUtils.isBlank(status)) this.jobStatus = STATUS_BEGIN;
        else {
            List<WrapPeriodJobDto.Task> tasks = WrapPeriodJobDto.fromJson(status);
            if (Collections.isEmpty(tasks)) this.jobStatus = STATUS_BEGIN;
            else {
                String[] codes = stepsCode.split(",");
                if (tasks.stream().anyMatch(m -> m.status.equals(STATUS_ERROR))) this.jobStatus = STATUS_ERROR;
                else if (tasks.size() < codes.length) this.jobStatus = STATUS_RUNNING;
                else this.jobStatus = STATUS_END;
            }
        }

        return this;
    }
}
