package pwc.taxtech.atms.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.CommonConstants;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.entity.AccountMapping;
import pwc.taxtech.atms.entity.AccountMappingExample;
import pwc.taxtech.atms.entity.EnterpriseAccount;

@Service
public class AccountServiceImpl extends AbstractService {

    @Autowired
    EnterpriseAccountServiceImpl enterpriseAccountService;

    /**
     * 匹配标准科目到企业科目（按科目编码）
     *
     * @param enterpriseAccountSetId - 账套Id
     * @param stdAccountCode         - 标准科目编码
     * @param epAccountCode          - 企业科目编码
     * @param industryId             - 行业Id
     * @param organizationId         - 机构Id
     * @param bOverwrite             - 是否覆盖已有对应
     * @param acctLevel              - 科目级别
     */
    public void mapStdAccountByCode(String enterpriseAccountSetId, String stdAccountCode, String epAccountCode, String industryId, String organizationId,
                                    Boolean bOverwrite, Integer acctLevel) {

        //set default value
        if (bOverwrite == null) {
            bOverwrite = true;
        }
        if (acctLevel == null) {
            acctLevel = 0;
        }

        EnterpriseAccount epAccount = enterpriseAccountService.getEnterpriseAccount(epAccountCode, enterpriseAccountSetId);
        if (epAccount == null) {
            return;
        }

        //set stdAccountCode
        if (stdAccountCode == null || stdAccountCode.isEmpty()) {
            stdAccountCode = CommonConstants.NullStdCode;
        }

        if (!bOverwrite) {
            if (!isStdExists(enterpriseAccountSetId, epAccountCode, industryId, organizationId)) {
                deleteAccountMapping(epAccountCode, epAccount.getEnterpriseAccountSetId(), organizationId, industryId);
                updateStdAccountMapping(epAccountCode, epAccount.getEnterpriseAccountSetId(), stdAccountCode, industryId, organizationId);
            }
        } else {
            deleteAccountMapping(epAccountCode, epAccount.getEnterpriseAccountSetId(), organizationId, industryId);
            updateStdAccountMapping(epAccountCode, epAccount.getEnterpriseAccountSetId(), stdAccountCode, industryId, organizationId);
        }
    }

    /**
     * 删除对应关系
     *
     * @param epAccountCode
     * @param enterpriseAccountSetId
     * @param organizaionId
     * @param industryId
     */
    private void deleteAccountMapping(String epAccountCode, String enterpriseAccountSetId, String organizaionId,
                                      String industryId) {

        AccountMappingExample accountMappingExample = new AccountMappingExample();
        accountMappingExample.createCriteria()
                .andEnterpriseAccountCodeEqualTo(epAccountCode)
                .andEnterpriseAccountSetIdEqualTo(enterpriseAccountSetId)
                .andOrganizationIdEqualTo(organizaionId)
                .andIndustryIdEqualTo(industryId);

        accountMappingMapper.deleteByExample(accountMappingExample);
    }

    /**
     * 插入对应关系到对应关系表
     *
     * @param epAccountCode
     * @param enterpriseAccountSetId
     * @param stdAccountCode
     * @param industryId
     * @param enterpriseAccountId
     * @param organizationId
     */
    private void updateStdAccountMapping(String epAccountCode, String enterpriseAccountSetId,
                                         String stdAccountCode, String industryId, String organizationId) {

        AccountMapping accountMapping = new AccountMapping();
        accountMapping.setId(CommonUtils.getUUID());
        accountMapping.setEnterpriseAccountSetId(enterpriseAccountSetId);
        accountMapping.setEnterpriseAccountCode(epAccountCode);
        accountMapping.setStandardAccountCode(stdAccountCode);
        accountMapping.setIndustryId(industryId);
        accountMapping.setOrganizationId(organizationId);
        accountMappingMapper.insert(accountMapping);
    }

    private boolean isStdExists(String enterpriseAccountSetId, String enterpirseAccountCode, String industryId, String organizationId) {

        Long stdAccountsCount = customAccountMapper.countStandardAccounts(enterpriseAccountSetId, enterpirseAccountCode, industryId, organizationId);

        return stdAccountsCount >= 1;
    }

}
