package pwc.taxtech.atms.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.CommonConstants;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.OperateLogType;
import pwc.taxtech.atms.common.OperationAction;
import pwc.taxtech.atms.common.OperationModule;
import pwc.taxtech.atms.common.message.AreaMessage;
import pwc.taxtech.atms.common.message.AreaRegionMessage;
import pwc.taxtech.atms.common.message.LogMessage;
import pwc.taxtech.atms.dao.AreaMapper;
import pwc.taxtech.atms.dao.AreaRegionMapper;
import pwc.taxtech.atms.dao.RegionMapper;
import pwc.taxtech.atms.dto.OperationLogDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.UpdateLogParams;
import pwc.taxtech.atms.dto.arearegion.AreaRegionDto;
import pwc.taxtech.atms.dto.arearegion.AreaRegionInfo;
import pwc.taxtech.atms.dto.arearegion.CityInfo;
import pwc.taxtech.atms.entity.Area;
import pwc.taxtech.atms.entity.AreaExample;
import pwc.taxtech.atms.entity.AreaRegion;
import pwc.taxtech.atms.entity.AreaRegionExample;
import pwc.taxtech.atms.entity.Region;
import pwc.taxtech.atms.entity.RegionExample;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class AreaRegionServiceImpl {

    @Autowired
    private AreaRegionMapper areaRegionMapper;

    @Autowired
    private AreaMapper areaMapper;

    @Autowired
    private OperationLogServiceImpl operationLogService;
    @Autowired
    private AuthUserHelper authUserHelper;
    @Autowired
    private RegionMapper regionMapper;

    public List<AreaRegion> findAreaRegionsByArea(String areaId) {

        return areaRegionMapper.selectAreaRegionByAreaId(areaId);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * pwc.taxtech.atms.service.AreaRegionService#add(pwc.taxtech.atms.service.dto.
     * AreaRegionInfo)
     */
    @Transactional
    public OperationResultDto add(AreaRegionInfo areaRegionInfo) {

        // check area name is duplicate or not
        AreaExample areaExample = new AreaExample();
        areaExample.createCriteria().andNameEqualTo(areaRegionInfo.getName());
        if (areaMapper.countByExample(areaExample) > 0) {
            return new OperationResultDto<>(false, AreaRegionMessage.AreaRegionRepeat);
        }

        // save area
        Area area = new Area();
        area.setId(CommonUtils.getUUID());
        area.setName(areaRegionInfo.getName());
        area.setIsActive(CommonConstants.ACTIVE_STATUS);
        area.setParentId(areaRegionInfo.getParentId());
        areaMapper.insert(area);

        // save areaRegion
        List<CityInfo> cityList = areaRegionInfo.getCityList();
        if (cityList != null && !cityList.isEmpty()) {

            // province
            List<String> provinceIdList = cityList.stream().map(city -> city.getParentId()).distinct()
                    .collect(Collectors.toList());
            for (String provinceId : provinceIdList) {
                AreaRegion provinceRegion = new AreaRegion();
                provinceRegion.setId(CommonUtils.getUUID());
                provinceRegion.setAreaId(area.getId());
                provinceRegion.setRegionId(provinceId);
                areaRegionMapper.insert(provinceRegion);
            }
            // city
            for (CityInfo city : cityList) {
                AreaRegion cityRegion = new AreaRegion();
                cityRegion.setId(CommonUtils.getUUID());
                cityRegion.setAreaId(area.getId());
                cityRegion.setRegionId(city.getId());
                areaRegionMapper.insert(cityRegion);
            }
        }

        OperationLogDto operationLogDto = new OperationLogDto();
        operationLogDto.setOperationContent("");
        operationLogDto.setAction(OperationAction.New.value());
        operationLogDto.setOperationObject(area.getName());
        operationLogDto.setOriginalState("");
        operationLogDto.setUpdateState("");
        operationLogDto.setModule(OperationModule.BasicDataArea.value());
        operationLogDto.setComment(LogMessage.Add);
        operationLogDto.setLogType(OperateLogType.OperationLogBasicData.value());
        operationLogService.addOperationLog(operationLogDto);

        return new OperationResultDto<>(true);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * pwc.taxtech.atms.service.AreaRegionService#update(pwc.taxtech.atms.service.
     * dto.AreaRegionInfo)
     */
    @Transactional
    public OperationResultDto update(AreaRegionInfo areaRegionInfo) {

        // Find oringinal area entity
        Area originalArea = areaMapper.selectByPrimaryKey(areaRegionInfo.getId());
        Area updateToArea = new Area();
        updateToArea.setId(areaRegionInfo.getId());
        updateToArea.setName(areaRegionInfo.getName());

        if (originalArea == null) {
            return new OperationResultDto<>(false, AreaMessage.NoAreaRootData);
        }

        // area name change
        if (!originalArea.getName().equals(updateToArea.getName())) {
            // check area name is duplicate or not
            AreaExample areaExample = new AreaExample();
            areaExample.createCriteria().andNameEqualTo(updateToArea.getName()).andIdNotEqualTo(updateToArea.getId());

            if (areaMapper.countByExample(areaExample) > 0) {
                return new OperationResultDto<>(false, AreaRegionMessage.AreaRegionRepeat);
            }

            // update area
            areaMapper.updateByPrimaryKeySelective(updateToArea);
            updateToArea = areaMapper.selectByPrimaryKey(originalArea.getId());

            // add operation log
            UpdateLogParams updateLogParams = new UpdateLogParams();
            updateLogParams.setOperateLogType(OperateLogType.OperationLogBasicData.value());
            updateLogParams.setComment("");
            updateLogParams.setOperationModule(OperationModule.BasicDataArea.value());
            updateLogParams.setOperationContent("");
            updateLogParams.setOriginalState(originalArea);
            updateLogParams.setUpdateState(updateToArea);
            updateLogParams.setOperationUser(authUserHelper.getCurrentAuditor().get());
            updateLogParams.setOperationObject(updateToArea.getName());
            updateLogParams.setOperationAction(OperationAction.Update.value());
            operationLogService.updateDataAddLog(updateLogParams);
        }

        List<AreaRegion> orginalRegionList = findAreaRegionsByArea(originalArea.getId());
        String orginalCityNames = "";
        if (null != orginalRegionList && !orginalRegionList.isEmpty()) {
            orginalCityNames = StringUtils
                    .collectionToDelimitedString(
                            orginalRegionList.stream()
                                    .filter(areaRegion -> areaRegion.getRegion()
                                            .getLevelType() == CommonConstants.REGION_LEVELTYPE_CITY)
                                    .map(areaRegion -> areaRegion.getRegion().getName()).collect(Collectors.toList()),
                            ",");
        }
        List<CityInfo> updateToCityList = areaRegionInfo.getCityList();
        String updateToCityNames = "";
        if (null != updateToCityList && !updateToCityList.isEmpty()) {
            // updateToCityList.stream().sorted(Comparator.comparing(cityInfo ->
            // cityInfo.getId())).collect(Collectors.toList());
            updateToCityNames = StringUtils.collectionToDelimitedString(
                    updateToCityList.stream().sorted(Comparator.comparing(cityInfo -> cityInfo.getId()))
                            .map(cityInfo -> cityInfo.getName()).collect(Collectors.toList()),
                    ",");
        }

        // region change
        if (!orginalCityNames.equals(updateToCityNames)) {
            // delete first
            AreaRegionExample areaRegionExample = new AreaRegionExample();
            areaRegionExample.createCriteria().andAreaIdEqualTo(originalArea.getId());
            areaRegionMapper.deleteByExample(areaRegionExample);

            if (null != updateToCityList && !updateToCityList.isEmpty()) {

                // add province
                List<String> provinceIdList = updateToCityList.stream().map(city -> city.getParentId()).distinct()
                        .collect(Collectors.toList());
                for (String provinceId : provinceIdList) {
                    AreaRegion provinceRegion = new AreaRegion();
                    provinceRegion.setId(CommonUtils.getUUID());
                    provinceRegion.setAreaId(originalArea.getId());
                    provinceRegion.setRegionId(provinceId);
                    areaRegionMapper.insert(provinceRegion);
                }
                // city
                for (CityInfo city : updateToCityList) {
                    AreaRegion cityRegion = new AreaRegion();
                    cityRegion.setId(CommonUtils.getUUID());
                    cityRegion.setAreaId(originalArea.getId());
                    cityRegion.setRegionId(city.getId());
                    areaRegionMapper.insert(cityRegion);
                }
            }

            // add operation log
            OperationLogDto operationLogDto = new OperationLogDto();
            operationLogDto.setAction(OperationAction.Update.value());
            operationLogDto.setComment("");
            operationLogDto.setLogType(OperateLogType.OperationLogBasicData.value());
            operationLogDto.setModule(OperationModule.BasicDataArea.value());
            operationLogDto.setOperationContent(AreaMessage.AreaRegion);
            operationLogDto.setOperationObject(updateToArea.getName());
            operationLogDto.setOriginalState(orginalCityNames);
            operationLogDto.setUpdateState(updateToCityNames);
            operationLogService.addOperationLog(operationLogDto);
        }

        return new OperationResultDto(true);
    }

    public List<AreaRegionDto> getProvinces() {
        List<AreaRegionDto> provinces = getRegions(CommonConstants.REGION_LEVELTYPE_PROVINCE);
        return provinces;
    }

    public List<AreaRegionDto> getCities(String parentId) {
        List<AreaRegionDto> cities = getRegions(CommonConstants.REGION_LEVELTYPE_CITY);
        List<AreaRegionDto> query = cities.stream().filter(s -> s.getRegionParentId().equals(parentId))
                .collect(Collectors.toList());
        return query;

    }

    private List<AreaRegionDto> getRegions(int levelType) {
        RegionExample regionExample = new RegionExample();
        regionExample.createCriteria().andIsActiveEqualTo(CommonConstants.ACTIVE_STATUS).andLevelTypeEqualTo(levelType);
        List<Region> list = regionMapper.selectByExample(regionExample);
        return rotateAreaRegionList(list);

    }

    private List<AreaRegionDto> rotateAreaRegionList(List<Region> regionList) {
        List<AreaRegionDto> areaRegionDtoList = new ArrayList<>();
        if (regionList != null && !regionList.isEmpty()) {
            for (Region region : regionList) {
                areaRegionDtoList.add(rotateRegion(region));
            }
        }
        return areaRegionDtoList;
    }

    private AreaRegionDto rotateRegion(Region region) {
        AreaRegionDto areaRegionDto = new AreaRegionDto();
        areaRegionDto.setRegionId(region.getId());
        areaRegionDto.setRegionName(region.getName());
        areaRegionDto.setLevelType(region.getLevelType());
        areaRegionDto.setRegionParentId(region.getParentId());
        return areaRegionDto;
    }
}
