package pwc.taxtech.atms.service.impl;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.config.SystemConfig;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.dao.AccountMappingDao;
import pwc.taxtech.atms.dao.EnterpriseAccountSetOrgDao;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dao.StandardAccountMapper;
import pwc.taxtech.atms.dto.vatdto.TrialBalanceDto;
import pwc.taxtech.atms.dto.vatdto.TrialBalanceParam;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.vat.dao.GlBalanceMapper;
import pwc.taxtech.atms.vat.entity.GlBalance;
import pwc.taxtech.atms.vat.entity.GlBalanceExample;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class DataImportService extends BaseService {

    @Resource
    private GlBalanceMapper glBalanceMapper;
    @Resource
    private OrganizationMapper organizationMapper;
    @Resource
    private StandardAccountMapper standardAccountMapper;
    @Resource
    private SystemConfig systemConfig;
    @Resource
    private EnterpriseAccountSetOrgDao enterpriseAccountSetOrgDao;
    @Resource
    private AccountMappingDao accountMappingDao;

    /**
     * TB表查询
     *
     * @param param TrialBalanceParam
     * @return List<TrialBalanceDto>
     */
    public List<TrialBalanceDto> getBalanceDataForDisplay(TrialBalanceParam param) {
        if (!ObjectUtils.allNotNull(param, param.getFromPeriod(), param.getToPeriod()) || StringUtils.isBlank(param.getOrgId())) {
            return Collections.emptyList();
        }
        Organization organization = organizationMapper.selectByPrimaryKey(param.getOrgId());
        if (null == organization) {
            return Collections.emptyList();
        }
        Optional<EnterpriseAccountSetOrg> accountSetOrg = enterpriseAccountSetOrgDao.getByOrganization(organization.getId());
        if (!accountSetOrg.isPresent()) {
            return Collections.emptyList();
        }
        // mapping
        List<AccountMapping> mappingList = accountMappingDao.getAccountMapping(accountSetOrg.get().getEnterpriseAccountSetId(),
                organization.getId());
        List<String> etCodeList = mappingList.stream().map(AccountMapping::getEnterpriseAccountCode).distinct().collect(Collectors.toList());
        Map<String, List<AccountMapping>> mappingMap = mappingList.stream().collect(Collectors.groupingBy(AccountMapping::getStandardAccountCode));
        //standard account
        StandardAccountExample stdExample = new StandardAccountExample();
        stdExample.createCriteria().andIsLeafEqualTo(Boolean.TRUE);
        List<StandardAccount> stdAccountList = standardAccountMapper.selectByExample(stdExample);
        Calendar calendar = Calendar.getInstance();
        SimpleDateFormat dateFormat = new SimpleDateFormat(Constant.DateFormat.YEAR_MONTH);
        Map<String, TrialBalanceDto> resultStdMap = new HashMap<>();
        for (int i = param.getFromPeriod(); i <= param.getToPeriod(); i++) {
            calendar.set(Calendar.MONTH, i - 1);
            GlBalanceExample example = new GlBalanceExample();
            //科目段
            GlBalanceExample.Criteria c = example.createCriteria().andSegment1EqualTo(organization.getClientCode())
                    .andPeriodNameEqualTo(dateFormat.format(calendar.getTime()));
//            if(CollectionUtils.isNotEmpty(etCodeList)){
                c.andSegment3In(etCodeList);
//            }
            List<GlBalance> sg3List = glBalanceMapper.selectByExample(example);

            GlBalanceExample example2 = new GlBalanceExample();
            //明细段
            GlBalanceExample.Criteria c2 = example2.createCriteria().andSegment1EqualTo(organization.getClientCode())
                    .andPeriodNameEqualTo(dateFormat.format(calendar.getTime()));
//            if(CollectionUtils.isNotEmpty(etCodeList)){
                 c2.andSegment4In(etCodeList);
//            }
            List<GlBalance> sg4List = glBalanceMapper.selectByExample(example2);

            for (StandardAccount standardAccount : stdAccountList) {
                if (!resultStdMap.containsKey(standardAccount.getCode())) {
                    resultStdMap.put(standardAccount.getCode(), new TrialBalanceDto());
                }
//                if (!mappingMap.containsKey(standardAccount.getCode())) {
//                    continue;
//                }
                TrialBalanceDto dto = resultStdMap.get(standardAccount.getCode());
                dto.setChildren(Collections.emptyList());
                dto.setAcctCode(standardAccount.getCode());
                dto.setAccountName(standardAccount.getName());
                List<AccountMapping> tmp = mappingMap.get(standardAccount.getCode());
                if (CollectionUtils.isEmpty(tmp)) {
                    continue;
                }
                tmp.forEach(o -> {
                    dto.setCreditBal(add(dto.getCreditBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getPtdCr()));
                    dto.setBegCreditBal(add(dto.getBegCreditBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getBeginCrBalance()));
                    dto.setEndCreditBal(add(dto.getEndCreditBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getEndCrBalance()));
                    dto.setYearCreditBal(add(dto.getYearCreditBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getYtdCr()));
                    dto.setDebitBal(add(dto.getDebitBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getPtdDr()));
                    dto.setBegDebitBal(add(dto.getBegDebitBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getBeginDrBalance()));
                    dto.setEndDebitBal(add(dto.getEndDebitBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getEndDrBalance()));
                    dto.setYearDebitBal(add(dto.getYearDebitBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getYtdDr()));
                });
            }
        }

        return new ArrayList<>(resultStdMap.values()).stream().sorted(Comparator.comparing(TrialBalanceDto::getAcctCode))
                .collect(Collectors.toList());
    }

    private BigDecimal add(BigDecimal v1, BigDecimal v2) {
        v1 = null == v1 ? BigDecimal.ZERO : v1;
        v2 = null == v2 ? BigDecimal.ZERO : v2;
        return v1.add(v2);
    }

    private GlBalance getGlBalance(String code, List<GlBalance> seg3, List<GlBalance> seg4) {
        for (GlBalance balance : seg3) {
            if (StringUtils.equals(code, balance.getSegment3())) {
                return balance;
            }
        }

        for (GlBalance balance : seg4) {
            if (StringUtils.equals(code, balance.getSegment3())) {
                return balance;
            }
        }
        return new GlBalance();
    }

    /**
     * 计算总和
     *
     * @param list List<TrialBalanceDto>
     * @return TrialBalanceDto
     */
    public TrialBalanceDto calculateData(List<TrialBalanceDto> list) {
        TrialBalanceDto dto = new TrialBalanceDto();
        list.forEach(o -> {
            dto.setCreditBal(add(dto.getCreditBal(), o.getCreditBal()));
            dto.setBegCreditBal(add(dto.getBegCreditBal(), o.getBegCreditBal()));
            dto.setEndCreditBal(add(dto.getEndCreditBal(), o.getEndCreditBal()));
            dto.setYearCreditBal(add(dto.getYearCreditBal(), o.getYearCreditBal()));

            dto.setDebitBal(add(dto.getDebitBal(), o.getDebitBal()));
            dto.setBegDebitBal(add(dto.getBegDebitBal(), o.getBegDebitBal()));
            dto.setEndDebitBal(add(dto.getEndDebitBal(), o.getEndDebitBal()));
            dto.setYearDebitBal(add(dto.getYearDebitBal(), o.getYearDebitBal()));
        });
        return dto;
    }

}
