package pwc.taxtech.atms.vat.service.impl.report.functions;

import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceType;
import pwc.taxtech.atms.dpo.CellTemplatePerGroupDto;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.vat.entity.GlBalance;
import pwc.taxtech.atms.vat.entity.GlBalanceExample;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

public class DFFS extends FunctionBase implements FreeRefFunction {
    public DFFS(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length < 4) {
            return null;
        }

        int type = getIntParam(args[0], ec);
        String code = getStringParam(args[1], ec);
        int year = getIntParam(args[2], ec);
        int period = getIntParam(args[3], ec);

        String formulaExpression = "DFFS(" + type + ",\"" + code + "\","
                + year + "," + period + ")";
        logger.debug(formulaExpression);

        year = getYear(year);

        Integer yearOffset = 0;
        period = FormulaHelper.getPeriod(period, yearOffset, formulaContext);
        String orgId = formulaContext.getOrganizationId();
        year = year + yearOffset;

        List<ReportCellDataSourceDto> dataSource = Lists.newArrayList();
        if(type == 0){
            EnterpriseAccountSetOrgExample example = new EnterpriseAccountSetOrgExample();
            example.createCriteria().andEffectiveDateLessThanOrEqualTo(DateUtils.getNow()).andExpiredDateGreaterThanOrEqualTo(DateUtils.getNow()).andOrganizationIdEqualTo(orgId);
            List<EnterpriseAccountSetOrg> enterpriseAccountSetOrgs = SpringContextUtil.enterpriseAccountSetOrgMapper.selectByExample(example);
            if(CollectionUtils.isEmpty(enterpriseAccountSetOrgs)){
                return NumberEval.ZERO;
            }
            AccountMappingExample accountMappingExample = new AccountMappingExample();
            accountMappingExample.createCriteria().andOrganizationIdEqualTo(orgId).andEnterpriseAccountSetIdEqualTo(
                    enterpriseAccountSetOrgs.get(0).getEnterpriseAccountSetId()).andStandardAccountCodeEqualTo(code);
            List<AccountMapping> accountMappings = SpringContextUtil.accountMappingMapper.selectByExample(accountMappingExample);
            double result = 0;

            for(AccountMapping a : accountMappings){
                result += count(a.getEnterpriseAccountCode(), dataSource, period, year, orgId);
            }
            Long dataSoureId = saveDataSource(ec, Collections.singletonList(dataSource),
                    FormulaDataSourceDetailType.FormulaDataSourceDto,
                    new BigDecimal(result), period, formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
            saveFormulaBlock(period, ec, formulaExpression, new BigDecimal(result), dataSoureId, formulaContext.getProjectId());

           return new NumberEval(result);
        } else if(type == 1){

            double result = count(code, dataSource, period, year, orgId);
            Long dataSoureId = saveDataSource(ec, Collections.singletonList(dataSource),
                    FormulaDataSourceDetailType.FormulaDataSourceDto,
                    new BigDecimal(result), period, formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
            saveFormulaBlock(period, ec, formulaExpression, new BigDecimal(result), dataSoureId, formulaContext.getProjectId());
            return new NumberEval(result);
        }

        return NumberEval.ZERO;
    }

    private String periodName(int period, int year){
        return year + "-" + (period > 9 ? period : ("0" + period));
    }

    private double count(String code, List<ReportCellDataSourceDto> contain, int period, int year, String orgId){
        Organization organization = SpringContextUtil.organizationMapper.selectByPrimaryKey(orgId);
        GlBalanceExample glBalanceExample = new GlBalanceExample();
        GlBalanceExample.Criteria c1 = glBalanceExample.createCriteria().andSegment3EqualTo(code)
                .andPeriodNameEqualTo(periodName(period, year));
        GlBalanceExample.Criteria c2 = glBalanceExample.createCriteria().andSegment4EqualTo(code)
                .andPeriodNameEqualTo(periodName(period, year));
        if(organization != null){
            c1.andSegment1EqualTo(organization.getClientCode());
            c2.andSegment1EqualTo(organization.getClientCode());
        }
        glBalanceExample.or(c2);
        List<GlBalance> list = SpringContextUtil.glBalanceMapper.selectByExample(glBalanceExample);
        if(CollectionUtils.isEmpty(list)){
            return 0.0;
        }
        List<GlBalance> temp = list.stream().filter(x -> code.equalsIgnoreCase(x.getSegment3())).collect(Collectors.toList());
        if(CollectionUtils.isEmpty(temp)){
            temp = list;
        }

        for(GlBalance glBalance : temp){
            ReportCellDataSourceDto dto = new ReportCellDataSourceDto();
            dto.setAmount(glBalance.getPtdCr());
            dto.setPeriod(period);
            dto.setIsOnlyManualInput(Boolean.FALSE);
            dto.setName(Constant.DataSourceName.ReportDataSource);
            dto.setType(FormulaDataSourceType.TrialBalanceSource.getCode());
            contain.add(dto);
        }
        return temp.stream().mapToDouble(a -> a.getPtdCr().doubleValue()).sum();
    }
}
