package pwc.taxtech.atms.dao;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import pwc.taxtech.atms.MyMapper;
import pwc.taxtech.atms.dpo.CellTemplatePerGroupDto;
import pwc.taxtech.atms.dpo.GroupId;
import pwc.taxtech.atms.entity.Project;

import java.util.List;

@Mapper
public interface FormulaAdminMapper extends MyMapper {

    @Select("" +
            "SELECT  " +
            "  cell_template.CELL_TEMPLATE_ID AS cellTemplateID,  " +
            "  template.TEMPLATE_ID AS reportTemplateID,  " +
            "  template.template_group_id AS reportTemplateGroupID,  " +
            "  cell_template.row_index AS rowIndex,  " +
            "  cell_template.column_index AS columnIndex,  " +
            "  template.Code AS reportCode,  " +
            "  cell_template.column_name AS columnName,  " +
            "  cell_template.row_name AS rowName,  " +
            "  cell_template.data_type AS resultType   " +
            "FROM  " +
            "  period_template template  " +
            "  JOIN period_cell_template cell_template ON template.TEMPLATE_ID = cell_template.report_template_id   " +
            "WHERE  " +
            "  template.TEMPLATE_ID IN ( SELECT TEMPLATE_ID FROM period_template WHERE TEMPLATE_GROUP_ID = #{templateGroupId} " +
            "  AND IS_ACTIVE_ASSOCIATION = 1 AND CODE = #{code} )   " +
            "  AND ROW_INDEX = #{rowIndex}   " +
            "  AND COLUMN_INDEX = #{columnIndex}   " +
            "  AND template.PROJECT_ID = #{projectId} " +
            "  AND template.PERIOD = #{period}   " +
            "  AND cell_template.PROJECT_ID = #{projectId}   " +
            "  AND cell_template.PERIOD = #{period}" +
            "")
    List<CellTemplatePerGroupDto> getCellTemplatePerGroupDto(@Param("templateGroupId") Long templateGroupId,
                                                             @Param("code") String code,
                                                             @Param("rowIndex") Integer rowIndex,
                                                             @Param("columnIndex") Integer columnIndex,
                                                             @Param("projectId") String projectId,
                                                             @Param("period") Integer period);


    @Select("SELECT  " +
            "    rule.template_group_id as groupId, rule.is_default as isDefault " +
            "FROM " +
            "    Project p, " +
            "    tax_payer_report_rule rule " +
            "WHERE " +
            "    (p.OrganizationID = rule.organization_id " +
            "        OR rule.is_default = 1) " +
            "        AND p.id = #{projectId}")
    List<GroupId> getTemplateGroupId(@Param("projectId") String projectId);

    @Select("SELECT  " +
            "    p.ID " +
            "FROM " +
            "    Project p " +
            "        JOIN " +
            "    Project_Service_Type ps ON p.id = ps.Project_ID " +
            "WHERE " +
            "    ps.Service_Type_ID IN ('2' , '3') " +
            "        AND p.Year = #{year} " +
            "        AND p.Organization_ID = #{orgId} " +
            "        AND p.Is_Active = 1 " +
            " AND ROWNUM=1"+
            "ORDER BY ps.Service_Type_ID ")
    String getPastProjectId(@Param("year") int year, @Param("orgId") String organizationId);

    @Select("" +
            "SELECT  " +
            "  p.ID AS id,  " +
            "  p.CODE AS code,  " +
            "  p.NAME AS name,  " +
            "  p.ORGANIZATION_ID AS organizationId   " +
            "FROM  " +
            "  PROJECT p  " +
            "  JOIN ( SELECT CODE, ORGANIZATION_ID FROM PROJECT WHERE ID = #{projectId}  ) t ON p.CODE = t.CODE   " +
            "  AND p.ORGANIZATION_ID = t.ORGANIZATION_ID AND p.YEAR = #{year}" +
            "")
    Project getFixedProject(@Param("projectId") String projectId, @Param("year") Integer year);
}
