﻿commonModule.controller('orgSelectorController', ['$scope', 'orgService', '$translate', 'userService', 'loginContext',
function ($scope, orgService, $translate, userService, loginContext) {

    var NoOptions = {
        'id': constant.organization.parentIdNull,
        'name': $translate.instant('OrgSelectNoOption')
    };
    var selectorg = $translate.instant('SelectOrg');
    $scope.pleaseSelect =  $translate.instant('PleaseSelect') ;
    $scope.init = function () {

        //设置每一项的默认最大长度
        $scope.itemMaxLength = angular.isDefined($scope.itemMaxLength) ? $scope.itemMaxLength : constant.organization.defaultMaxLength;

        $scope.isRequire = angular.isDefined($scope.itemMaxLength) ? $scope.isRequire : true;

        $scope.isAdminUser = false;
        var useType = $scope.isShowAll ? 1 : 0;
        orgService.getOrgList(useType).success(function (data) {
            if (data && data.length > 0) {
                if ($scope.selectNoOption) {
                    data.splice(0, 0, NoOptions);
                }

                if (!$scope.queryOrganizationCode && !$scope.editOrganizationCode) {

                    // 设置机构树和默认值
                    setDataAndFirstDefault(data);

                    setSelectOrg();
                } else {
                    getUserOrganzationPermission(data);
                }
            }
        });
    };


    $scope.selectedOrg = null;

    $scope.getSelectedOrgDisplayText = function () {
        if ($scope.selectedOrgId != null) {
            return $scope.selectedOrgName;
        } else {
            return '';
        }
    };

    $scope.selectOrg = function (org) {
        $scope.selectedOrgName = org.name;
        $scope.selectedOrgId = org.id;
        $scope.componentSelectedOrg = org;

        $('.dropdown').removeClass('open');

        $("#selectedOrgName").val(org.name);
        if ($("#selectedOrgName").val() != null && $("#selectedOrgName").val() != selectorg && $("#selectedOrgName").val() != '') {
            $("#selectedOrgName-error").hide();
        } else {
            $("#selectedOrgName-error").show();
        }
    };

    $scope.loadParentData = function () {
        $scope.loadData();
    };

    $scope.getNumber = function (num) {
        if (!num) {
            num = 0;
        }

        return new Array(num);
    }

    var findOrg = function (orgid, data) {
        if (data && data.length > 0) {
            var target = _.find(data, function (num) {
                return num.id === orgid
            });

            if (target) {
                return target;
            }

            for (var i = 0; i < data.length - 1; i++) {
                var row = data[i];
                target = findOrg(orgid, row.subOrgs);
                if (target) {
                    return target;
                }
            }
        }

        return null;
    };

    var setSelectOrg = function () {
        if ($scope.selectedOrgId && !$scope.selectedOrgName) {
            var target = findOrg($scope.selectedOrgId, $scope.orgList);
            if (target) {
                $scope.selectOrg(target);
            }
        }
    };

    $scope.$watch('selectedOrgId', function (newValue, oldValue) {
        if (newValue !== oldValue) {

            if ($scope.componentSelectedOrg) {
                if ($scope.isAdminUser) {
                    $scope.componentSelectedOrg.isHaveQueryPermission = true;
                    $scope.componentSelectedOrg.isHaveEditPermission = true;
                }
            }

            $scope.loadParentData();
        }

        // 强制重新取数据
        if ($scope.componentSelectedOrg && $scope.componentSelectedOrg.forceRefresh) {
            $scope.init();
        }
    });

    // 获取所有机构列表
    var getAllOrganizationList = function (orgList) {
        var showIDList = [];
        orgList.forEach(function (row) {
            showIDList.push(row);
            if (row.subOrgs && row.subOrgs.length > 0) {
                var tempIDList = getAllOrganizationList(row.subOrgs);
                if (tempIDList && tempIDList.length > 0) {
                    showIDList = _.union(showIDList, tempIDList);
                }
            }
        });

        return showIDList;
    };

    // 设置机构权限
    var setOrgPermission = function (orgList, userPermission) {

        orgList.forEach(function (row) {
            row.isHaveQueryPermission = window.PWC.isHaveOrganizationPermission(row.id, $scope.queryOrganizationCode, userPermission);
            row.isHaveEditPermission = window.PWC.isHaveOrganizationPermission(row.id, $scope.editOrganizationCode, userPermission);
        });

        return orgList;
    };

    // 获取父节点
    var getParentID = function (current, allList, orgDtoList) {
        if (current.parentID === null) {
            return null;
        }

        var findParent = _.find(allList, function (num) {
            return num.id === current.parentID;
        });

        var parent = _.find(orgDtoList, function (num) {
            return num.id === findParent.id;
        });

        if (parent != null) {
            return parent.id;
        } else {
            return getParentID(findParent, allList, orgDtoList);
        }
    };

    // tree 数据
    var buildOrgTree = function (or, orgList) {

        var orgItem = or;
        var selfDimensionIndex = 1;

        var sublist = _.filter(orgList, function (org) {
            return org.parentID === or.id;
        });

        var children = [];

        if (sublist != null && sublist.length > 0) {
            sublist.forEach(function (org) {
                var temp = buildOrgTree(org);
                children.push(temp);
            });
        }

        orgItem.subOrgs = children;

        return orgItem;
    };


    // 设置第一个机构为默认选择机构
    var setDataAndFirstDefault = function (orgTreeList) {
        $scope.orgList = orgTreeList;
        if ($scope.isFirstDefault) {
            if (orgTreeList && orgTreeList.length > 0) {
                $scope.selectedOrgId = orgTreeList[0].id;
                $scope.selectedOrgName = orgTreeList[0].name;
                $scope.componentSelectedOrg = orgTreeList[0];
            } else {
                $scope.selectedOrgId = null;
                $scope.selectedOrgName = null;
                $scope.componentSelectedOrg = {};
            }
        }
    };

    // 获取设置权限后的机构树
    var getUserOrganzationTreeList = function (orgTreeList, userPermission) {

        if (userPermission.isAdmin) {
            $scope.isAdminUser = true;

            // 默认选中第一条数据
            setDataAndFirstDefault(orgTreeList);

        } else {

            // 转换成机构列表
            var allOrgList = getAllOrganizationList(orgTreeList);

            // 设置机构权限
            var setOrgPermissionList = setOrgPermission(allOrgList, userPermission);

            // 获取有查看或者编辑权限的机构
            var showList = _.filter(setOrgPermissionList, function (row) {
                return row.isHaveQueryPermission || row.isHaveEditPermission;
            });

            // 重新设置上级节点
            for (var i = 0; i < showList.length; i++) {
                var item = showList[i];
                if (item.parentID === null) {
                    continue;
                }

                var parentNode = _.find(showList, function (num) {
                    return num.id === item.parentID
                });

                if (parentNode) {
                    continue;
                }

                item.parentID = getParentID(item, setOrgPermissionList, showList);
            }

            // 构造树形结构
            // 获取所有根节点的机构
            var idList = _.pluck(showList, 'id');
            var topNodeList = _.filter(showList, function (org) {
                return idList.indexOf(org.parentID) === -1;
            });

            // 构造树形列表
            var orglist = [];
            topNodeList.forEach(function (org) {
                var temp = buildOrgTree(org, showList);
                orglist.push(temp);
            });

            // 设置机构树和默认值
            setDataAndFirstDefault(orglist);
        }

    };

    // 获取用户机构权限
    var getUserOrganzationPermission = function (orgData) {
        userService.getUserPermission(loginContext.userName).success(function (userPermission) {
            getUserOrganzationTreeList(orgData, userPermission);
            setSelectOrg();
        });
    };

    $scope.init();

    $scope.isOpen = false;
    $scope.closeDropdown = function ($event) {
        var elem = $event.originalEvent.srcElement;
        if ($(elem).hasClass("fa")) {
            $scope.isOpen = true;
            $event.preventDefault();
            $event.stopPropagation();
        }
    }
}
]);