﻿commonModule.
controller('permissionSelectorController', ['$scope', 'menuService', 'permissionService', '$translate',
    function ($scope, menuService, permissionService, $translate) {

        $scope.serviceType = constant.serviceType.VAT;
        $scope.selectedItemData = [];

        $scope.$watch('selectedPermissionNames', function (newValue, oldValue) {
            //先全部清空
            $scope.widgetInstance.unselectAll();
            $scope.selectedItemData = [];
        });

        var addSelectedItemData = function (node) {

            $scope.selectedItemData.push(node.itemData);

            if (node.children && node.children.length > 0) {
                node.children.forEach(function (item) {
                    $scope.selectedItemData.push(item.itemData);
                    addSelectedItemData(item);
                });
            }
        };

        var removeSelectedItemData = function (node) {

             $scope.selectedItemData = _.without($scope.selectedItemData, _.find($scope.selectedItemData, function (item) {
                            return item.id === node.key
                        }));

            if (node.children && node.children.length > 0) {
                node.children.forEach(function (item) {
                      $scope.selectedItemData = _.without($scope.selectedItemData, _.find($scope.selectedItemData, function (sub) {
                          return sub.id === item.key
                        }));
                    removeSelectedItemData(item);
                });
            }
        };

        $scope.searchOptions = {
            bindingOptions: {
                value: "searchValue"
            },
            placeholder: $translate.instant('Search'),
            width: '92%',
            mode: "search",
            valueChangeEvent: "keyup"
        };

        //初始化dx-tree-view结构
        var loadAllPermissionTree = function () {
            $scope.allPermissionTreeViewOptions = {
                bindingOptions: {
                    dataSource: 'allPermissionTreeViewData',
                    searchValue: 'searchValue',
                    expandAllEnabled: 'expandAll'
                },
                dataStructure: "plain",
                selection: {
                    mode: "multiple"
                },
                loadPanel: {
                    enabled: true
                },
                scrolling: {
                    mode: "virtual"
                },
                keyExpr: "id",
                showRowLines: true,
                showColumnLines: true,
                rowAlternationEnabled: true,
                showBorders: true,
                selectAllText: $translate.instant('SelectAll'),
                scrollDirection: 'vertical',  //Accepted Values: 'vertical' | 'horizontal' | 'both'
                selectNodesRecursive: true,  //级联选择
                showCheckBoxesMode: 'normal',  //Accepted Values: 'none' | 'normal' | 'selectAll'
                onInitialized: function (e) {
                    $scope.widgetInstance = e.component;
                    // $scope.widgetInstance.selectItem($scope.itemElement.id);
                },
                onItemSelectionChanged: function (e) {
                    var node = e.node;
                    var relyOnCodesStr = node.itemData.relyOnCodes;

                    //如果是选中的状态，将其添加的选中的列表中，否则移除
                    if (node.selected) {
                        addSelectedItemData(node);
                    } else {
                        removeSelectedItemData(node);
                    }

                    if (relyOnCodesStr !== null && relyOnCodesStr !== undefined) {
                        var relyOnCodesArray = relyOnCodesStr.split(',');
                        if (relyOnCodesArray && relyOnCodesArray.length > 0) {

                            //提取出统一层级的属性为数组
                            var relyOnCodeList = _.map(node.parent.children, function (item) {
                                return {
                                    'key': item.key,
                                    'code': item.itemData.relyOnCodes,
                                    'selected': item.itemData.selected
                                };
                            });

                            node.parent.children.forEach(function (child) {

                                if (!node.selected) {
                                    var filtered = _.filter(relyOnCodeList, function (item) {
                                        return item.code == child.itemData.code && item.selected !== node.selected;
                                    });

                                    if (filtered && filtered.length >= 0) return;
                                }

                                //https://www.devexpress.com/Support/Center/Question/Details/T499903/treeview-selectitem-method-not-working
                                if (relyOnCodesArray.indexOf(child.itemData.code) >= 0) {
                                    if (node.selected) {
                                        $scope.widgetInstance.selectItem(child.key);
                                    }
                                    else {
                                        $scope.widgetInstance.unselectItem(child.key);
                                    }
                                }
                            });
                        }
                    } else {

                        if (node.selected) {
                            //返回选中的keys
                            $scope.selectedPermissionCodes = $scope.widgetInstance.getSelectedNodesKeys();
                            return;
                        }
                            //比如手动取消查看的权限，那么此时修改等权限也应该做相应的改动
                            var codeList = _.map(node.parent.children, function (item) {
                                return {
                                    'key': item.key,
                                    'selected': item.selected,
                                    'relyOnCodes': item.itemData.relyOnCodes
                            };
                        });

                        var filtered = _.filter(codeList, function (item) {
                            return item.relyOnCodes !== null && item.relyOnCodes.indexOf(node.itemData.code) >= 0 && item.selected == true;
                        });

                            //unchecked 
                            _.each(filtered, function (item) {
                                $scope.widgetInstance.unselectItem(item.key);
                        });
                    }
                    
                    //返回选中的keys
                    $scope.selectedPermissionCodes = $scope.widgetInstance.getSelectedNodesKeys();
                }
            };
        };

        ///初始化权限列表
        var initPermission = function () {
            permissionService.getAllPermissions($scope.serviceType).success(function (data) {
                $scope.allPermissionTreeViewData = data;

                //先全部清空
                $scope.widgetInstance.unselectAll();
            });
        };

        //获取选中的名字并且在textbox显示
        $scope.getSelectedOrgDisplayText = function () {
            var selectedRoleNameList = _.pluck($scope.selectedItemData, 'text');
            var unique = _.uniq(selectedRoleNameList);
            return unique.join(',');
        };

        (function () {

            loadAllPermissionTree();
            initPermission();
        })();

    }
]);
