﻿cacheModule.factory('signalRSvc', ['$log', 'CacheFactory', 'loginContext','cacheService','notify','$translate',
    function ($log, CacheFactory, loginContext, cacheService, notify, $translate) {

    'use strict';
    $log.debug('signalr-cache.run()...');

    return {
        initialize: function () {

            //****************************************************************************************************************
            // refer to https://docs.microsoft.com/en-us/aspnet/signalr/overview/guide-to-the-api/hubs-api-guide-javascript-client
            // The following is for generated proxy,if you want to register multiple event handler
            // for a client method that the server calls, you can't use the generated proxy
            //****************************************************************************************************************
             
            var cacheName = constant.cache.cacheName;
            var hubName = constant.cache.hubName; 
            var cacheUpdateMessage = constant.cache.cacheUpdateMessage;

            var connection = $.hubConnection(loginContext.apiHost);
            var cacheHub = connection.createHubProxy(hubName);
            cacheHub.on('updateSignalrCache', function (data) {
                updateCache(data);
            });

            cacheHub.on('updateUserOnlineCount', function (count) {
                console.info('online user count New:', count);
            });
            
            connection.start().done(function () {
                console.log('Now connected -webapi , connection ID=' + connection.id);
            }).fail(function (error) {
                console.log('Could not connect - webapi', error);
            });

            //用户修改数据之后通过SignalR让在线用户及时更新缓存
            var updateCache = function (data) {
                 
                //data对应服务端的dto对象ClientCacheDto 
                var dataCache = CacheFactory.get(cacheName); 
                var cacheKey = data.CacheKey;
                var cacheTime = data.CacheTime;
                
                console.log('signal update triggerred to remove expired cache...,cacheKey:'+cacheKey);
                
                var keys = dataCache.keys();
                _.each(keys, function (key) {
                    if (key.toLowerCase().indexOf(cacheKey.toLowerCase()) >= 0) {
                        var removeKey = dataCache.remove(key);
                        console.log(key + ' was removed');
                    }
                });
                 
                notify({
                    message: cacheUpdateMessage,
                    position: constant.cache.notify.position,
                    classes: constant.cache.notify.classes
                });
            }
        }
    };
}]);

